/*
 * *************************************************
 * Copyright (c) 2018 QMTH. All Rights Reserved.
 * Created by Deason on 2018-08-01 15:17:10.
 * *************************************************
 */

package cn.com.qmth.examcloud.app.core.config;

import cn.com.qmth.examcloud.app.model.Constants;
import cn.com.qmth.examcloud.app.model.LoginInfo;
import cn.com.qmth.examcloud.app.model.Result;
import cn.com.qmth.examcloud.app.model.UserInfo;
import cn.com.qmth.examcloud.app.service.UserAuthService;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.HttpStatus;

import javax.servlet.FilterConfig;
import javax.servlet.*;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Date;

import static cn.com.qmth.examcloud.app.model.Constants.PLATFORM_SESSION_EXPIRE_TIME;

/**
 * 认证信息过滤器
 *
 * @author: fengdesheng
 * @since: 2018/8/1
 */
public class TokenFilter implements Filter {
    private final static Logger log = LoggerFactory.getLogger(TokenFilter.class);
    private UserAuthService userAuthService;

    @Override
    public void init(FilterConfig filterConfig) throws ServletException {
        log.debug("init");
    }

    @Override
    public void destroy() {
        log.debug("destroy");
    }

    @Override
    public void doFilter(ServletRequest servletRequest, ServletResponse servletResponse, FilterChain filterChain)
            throws IOException, ServletException {
        HttpServletRequest request = (HttpServletRequest) servletRequest;

        //移动端设备请求必须带“设备编号”参数
        /*String deviceId = request.getHeader(Constants.PARAM_DEVICE_ID);
        if (StringUtils.isBlank(deviceId)) {
            Result result = new Result().error("[APP] deviceId must be not empty.");
            HttpServletResponse response = (HttpServletResponse) servletResponse;
            this.render(response, result.toString());
            return;
        }*/

        CustomHttpServletRequest customRequest = this.initCustomRequest(request);
        if (customRequest != null) {
            filterChain.doFilter(customRequest, servletResponse);
            return;
        }
        filterChain.doFilter(servletRequest, servletResponse);
    }

    private CustomHttpServletRequest initCustomRequest(HttpServletRequest request) {
        String appKey = request.getHeader(Constants.PARAM_APP_KEY);
        String appToken = request.getHeader(Constants.PARAM_APP_TOKEN);
        if (StringUtils.isBlank(appKey) || StringUtils.isBlank(appToken)) {
            //key,token为空，则不用处理
            return null;
        }

        //通过key获取原始登录信息
        LoginInfo loginInfo = userAuthService.getLoginInfo(appKey);
        if (loginInfo == null) {
            //原始登录信息为空，则代表尚未登录不用处理
            return null;
        }

        //同一个账号不同移动端设备，支持互踢
        if (!appToken.equals(loginInfo.getAppToken())) {
            //App Token参数值无效
            return null;
        }

        //处理已登录信息
        CustomHttpServletRequest customRequest = new CustomHttpServletRequest(request);
        boolean isAllow = this.filterAccessUrl(request.getServletPath());
        if (!isAllow) {
            if (loginInfo.hasExpired(PLATFORM_SESSION_EXPIRE_TIME)) {
                //判断原始登录Token是否在有效时间内，否则自动登录续期
                this.reLogin(loginInfo);
            }
        }

        //将App Token替换为原始登录Token到request请求中
        customRequest.addHeader(Constants.PARAM_APP_KEY, loginInfo.getKey());
        customRequest.addHeader(Constants.PARAM_APP_TOKEN, loginInfo.getToken());

        //初始化内部接口请求鉴权
        userAuthService.initRequestTrace();
        return customRequest;
    }

    private void reLogin(LoginInfo loginInfo) {
        try {
            Result<UserInfo> result = userAuthService.login(loginInfo);
            UserInfo userInfo = result.getData();
            if (userInfo == null) {
                log.debug("reLogin failed,Maybe user's password has changed.");
                return;
            }
            //登录成功后缓存新的Token信息
            log.info(String.format("key:%s newToken:%s", userInfo.getKey(), userInfo.getToken()));
            loginInfo.setUserName(userInfo.getDisplayName());
            loginInfo.setToken(userInfo.getToken());
            loginInfo.setCreateTime(new Date());
            userAuthService.cacheLoginInfo(loginInfo, userInfo.getKey());
        } catch (Exception e) {
            log.error(e.getMessage());
        }
    }

    private boolean filterAccessUrl(String url) {
        //过滤不用处理的访问地址
        if (url.contains("/user/logout")) {
            return true;
        }
        return false;
    }

    private void render(HttpServletResponse response, String json) {
        PrintWriter out = null;
        try {
            response.setStatus(HttpStatus.OK.value());
            response.setCharacterEncoding("UTF-8");
            response.setContentType("application/json;charset=UTF-8");
            out = response.getWriter();
            out.print(json);
        } catch (IOException e) {
            log.error(e.getMessage(), e);
        } finally {
            if (out != null) {
                out.close();
            }
        }
    }

    public void setUserAuthService(UserAuthService userAuthService) {
        this.userAuthService = userAuthService;
    }

}